/*contiene la definizione di albero binario (di interi), i prototipi delle
 * funzioni e procedure su alberi binari:
 * + costruzione di un albero binario inserito da input
 * + visita/stampa di un albero
 * + calcolo del numero di elementi di un livello h
 * + calcolo del numero delle foglie dell'albero
 * + stampa in output indentata di un albero binario
 * + verifica se un albero binario sia o meno completo
 * + trasformazione di un albero nel suo speculare
 * + verifica se l'albero e' vuoto*/
#ifndef ALBERI_H
#define ALBERI_H
#include <stdio.h>
#include<stdlib.h>

struct nodoalbero{
	struct nodoalbero *right;
	int elem;
	struct nodoalbero *left;
};

struct answer{
	int completo;
	int pieno;
	int altezza;
};
typedef struct nodoalbero TREENODE;
typedef TREENODE *ALBERO;
typedef struct answer RISPOSTA;
typedef RISPOSTA *RISPOSTAPTR;

/*Costruzione di un albero Binario inserito da input. Si assume che gli interi
 * contenuti nei nodi dell'albero siano inseriti da input in preordine
 * Il formato dell'input saranno coppie di interi in cui il primo valore
 * corrisponde al valore da inserire nel nodo, il secondo valore indica il
 * numero dei suoi figli (0 nessun figlio, 1 solo il figlio sinistro, -1 solo
 * il figlio destro, 2 entrambi i figli). I nodi sono inseriti i preordine (prima la
 * radice), quindi i nodi del sottoalbero sinistro, quindi i nodi del sottoalbero
 * destro. Ad esempio:
 * 3 2
 * 1 0
 * 4 2
 * 5 0
 * 7 0
 *
 * costruisce l'albero binario
 *    3
 *  1    4
 *     5   7
 *  La seguente procedura costruisce l'albero binario */

ALBERO creaalbero();

/*Verifica se l'albero e' vuoto*/
int alberovuoto(ALBERO);

/*stampa in inorder i valori contenuti nell'albero*/
void stampainalbero(ALBERO);

/*calcola il numero di nodi dell'albero al livello specificato*/
int numelemlivello (ALBERO, int, int);

/*calcola il numero di foglie dell'albero*/
int numfoglie(ALBERO);

/*stampa l'albero indentato*/
void stampaindentata(ALBERO, int);

/*verifica se un albero binario e' completo. Restituisce una struttura il cui campo completo contiene 1 se cio' e' vero
 * i campi pieno e altezza contengono rispettivamente 1/0 a seconda che l'albero sia pieno o meno, e l'altezza
 * dell'albero */
RISPOSTAPTR completo(ALBERO);

/*trasforma un albero nel suo speculare*/
void speculare (ALBERO);
#endif
